package com.example.switchupdate.client.switchcomp;

import java.util.ArrayList;

import com.google.gwt.animation.client.Animation;
import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Document;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.dom.client.Touch;
import com.google.gwt.event.dom.client.BlurEvent;
import com.google.gwt.event.dom.client.BlurHandler;
import com.google.gwt.event.dom.client.FocusEvent;
import com.google.gwt.event.dom.client.FocusHandler;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.event.dom.client.MouseDownEvent;
import com.google.gwt.event.dom.client.MouseDownHandler;
import com.google.gwt.event.dom.client.MouseMoveEvent;
import com.google.gwt.event.dom.client.MouseMoveHandler;
import com.google.gwt.event.dom.client.MouseUpEvent;
import com.google.gwt.event.dom.client.MouseUpHandler;
import com.google.gwt.event.dom.client.TouchCancelEvent;
import com.google.gwt.event.dom.client.TouchCancelHandler;
import com.google.gwt.event.dom.client.TouchEndEvent;
import com.google.gwt.event.dom.client.TouchEndHandler;
import com.google.gwt.event.dom.client.TouchMoveEvent;
import com.google.gwt.event.dom.client.TouchMoveHandler;
import com.google.gwt.event.dom.client.TouchStartEvent;
import com.google.gwt.event.dom.client.TouchStartHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.FocusWidget;
import com.google.gwt.user.client.ui.HasAnimation;

// TODO extend any GWT Widget
public class SwitchCompWidget extends FocusWidget implements KeyUpHandler,
MouseDownHandler, MouseUpHandler, MouseMoveHandler, FocusHandler,
BlurHandler, HasAnimation, TouchStartHandler, TouchEndHandler,
TouchMoveHandler, TouchCancelHandler {

	/** Set the CSS class name to allow styling. */
    public static final String CLASSNAME = "v-switch";
    private final int DRAG_THRESHOLD_PIXELS = 10;
    private final int ANIMATION_DURATION_MS = 300;

    /** The client side widget identifier */
    protected String paintableId;


    private Element slider;
    public boolean value;
    private boolean mouseDown;
    private int unvisiblePartWidth = -1;
    private final DragInformation dragInfo = new DragInformation();

    private boolean animated;
    private int tabIndex;
    private ArrayList<HandlerRegistration> handlers;

	public SwitchCompWidget() {
		setElement(Document.get().createDivElement());
		animated = false;
        // This method call of the Paintable interface sets the component
        // style name in DOM tree
        setStyleName(CLASSNAME);

        // build the DOM
        slider = Document.get().createDivElement();
        slider.setClassName(CLASSNAME + "-" + "slider");
        getElement().appendChild(slider);
        addHandlers();
	}

	private void addHandlers() {
        handlers = new ArrayList<HandlerRegistration>();
        handlers.add(addKeyUpHandler(this));
        handlers.add(addMouseMoveHandler(this));
        handlers.add(addMouseDownHandler(this));
        handlers.add(addMouseUpHandler(this));
        handlers.add(addFocusHandler(this));
        handlers.add(addBlurHandler(this));

        handlers.add(addTouchStartHandler(this));
        handlers.add(addTouchEndHandler(this));
        handlers.add(addTouchCancelHandler(this));
        handlers.add(addTouchMoveHandler(this));
    }
	
	
	public void initState(){
		int width = this.getElement().getClientWidth();
        int sliderWidth = this.slider.getClientWidth();
        unvisiblePartWidth = sliderWidth - width;
        GWT.log(unvisiblePartWidth+" "+sliderWidth+" "+width);
        boolean skipAnimation = true;
        valprev = value;
        updateVisibleState(skipAnimation);
	}
	
	
	@Override
    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);

        if (!enabled) {
            super.setTabIndex(-1);
        } else {
            super.setTabIndex(tabIndex);
        }
    }

    @Override
    public void setTabIndex(int index) {
        super.setTabIndex(index);
        tabIndex = index;
    }

    public void setValue(boolean value, NativeEvent event) {
        if (this.value == value) {
            return;
        }

        this.value = value;

        // update the UI
        updateVisibleState();

        if (event != null) {
            // Update the server state if the value change was initiated from an
            // UI event (event != null).
            if (paintableId == null) {
                return;
            }

//            MouseEventDetails details = new MouseEventDetails(event,
//                    getElement());
//            client.updateVariable(paintableId, "mousedetails",
//                    details.serialize(), false);
//
//            client.updateVariable(paintableId, "state", this.value, immediate);
        }
    }

    private void updateVisibleState() {
        updateVisibleState(false);
    }

    private boolean valprev;
    private void updateVisibleState(boolean skipAnimation) {
        final int targetLeft = (value ? 0 : -unvisiblePartWidth);

        if (!isAnimationEnabled() || skipAnimation) {
            slider.getStyle().setProperty("left", targetLeft + "px");
            updateStyleName();
        } else {
            Animation a = new Animation() {

                @Override
                protected void onUpdate(double progress) {
                    int currentLeft = slider.getOffsetLeft();
                    int newLeft = (int) (currentLeft + (progress * (targetLeft - currentLeft)));
                    slider.getStyle().setProperty("left", newLeft + "px");
                }

                @Override
                protected void onComplete() {
                    super.onComplete();
                    updateStyleName();
                }
            };
            a.run(ANIMATION_DURATION_MS);
        }
        if(valprev != value){
        	firevalchange(value);
        	valprev = value;
        }
    }

    private void updateStyleName() {
        if (value) {
            addStyleName("on");
            removeStyleName("off");
        } else {
            addStyleName("off");
            removeStyleName("on");
        }
    }
	

	public void onKeyUp(KeyUpEvent event) {
        if (event.getNativeKeyCode() == 32) {
            // 32 = space bar
            setValue(!value, event.getNativeEvent());
        }
    }

    public void onMouseDown(MouseDownEvent event) {
        handleMouseDown(event.getScreenX());
        event.preventDefault();
    }

    private void handleMouseDown(int clientX) {
        mouseDown = true;
        dragInfo.setDragStartX(clientX);
        dragInfo.setDragStartOffsetLeft(slider.getOffsetLeft());
    }

    public void onMouseUp(MouseUpEvent event) {
        handleMouseUp(event.getNativeEvent());
    }

    private void handleMouseUp(NativeEvent event) {
        if (!dragInfo.isDragging()) {
            setValue(!value, event);
        } else {
            if (slider.getOffsetLeft() < (-unvisiblePartWidth / 2)) {
                setValue(false, event);
            } else {
                setValue(true, event);
            }
            updateVisibleState();
            DOM.releaseCapture(getElement());
        }

        mouseDown = false;
        dragInfo.setDragging(false); // not dragging anymore
    }

    public void onMouseMove(MouseMoveEvent event) {
        handleMouseMove(event.getScreenX());
    }

    private void handleMouseMove(int clientX) {
        if (mouseDown) {
            if (Math.abs(dragInfo.getDragDistanceX(clientX)) > DRAG_THRESHOLD_PIXELS) {
                dragInfo.setDragging(true);
                // Use capture to catch mouse events even if user
                // drags the mouse cursor out of the widget area.
                DOM.setCapture(getElement());
            }

            if (dragInfo.isDragging()) {
                int dragDistance = dragInfo.getDragDistanceX(clientX);

                // calculate new left position and
                // check for boundaries
                int left = dragInfo.getDragStartOffsetLeft() + dragDistance;
                if (left < -unvisiblePartWidth) {
                    left = -unvisiblePartWidth;
                } else if (left > 0) {
                    left = 0;
                }

                // set the CSS left
                slider.getStyle().setProperty("left", left + "px");
            }
        }
    }

    public void onFocus(FocusEvent event) {
        addStyleDependentName("focus");
    }

    public void onBlur(BlurEvent event) {
        removeStyleDependentName("focus");
    }

    public boolean isAnimationEnabled() {
        return animated;
    }

    public void setAnimationEnabled(boolean enable) {
        animated = enable;
    }

    public void onTouchCancel(TouchCancelEvent event) {
        handleMouseUp(event.getNativeEvent());
    }

    public void onTouchMove(TouchMoveEvent event) {
        Touch touch = event.getTouches().get(0).cast();
        handleMouseMove(touch.getPageX());
        event.preventDefault();
    }

    public void onTouchEnd(TouchEndEvent event) {
        handleMouseUp(event.getNativeEvent());
    }

    public void onTouchStart(TouchStartEvent event) {
        Touch touch = event.getTouches().get(0).cast();
        handleMouseDown(touch.getPageX());
        event.preventDefault();
    }
    
    ArrayList<ValChangeListener> listarr = new ArrayList<SwitchCompWidget.ValChangeListener>();
    public void addValChangeListener(ValChangeListener vcl) {
    	listarr.add(vcl);
	}
    
    public void removeValChangeListener(ValChangeListener vcl){
    	listarr.remove(vcl);
    }
    
    public void firevalchange(boolean newval){
    	for (int i = 0; i < listarr.size(); i++) {
			ValChangeListener vcl = listarr.get(i);
			vcl.valchange(newval);
		}
    }
    
    public interface ValChangeListener{
    	public void valchange(boolean newval);
	};
	
    }